import axios from 'axios';
import { getConfig, getAuthHeaders, getUrlParams } from '../utils/config.js';
/**
 * 掘金API基础HTTP客户端
 * 提供统一的请求处理、错误处理和重试机制
 */
export class JuejinApiClient {
    client;
    config = getConfig();
    maxRetries;
    retryDelay = 1000;
    constructor() {
        this.maxRetries = this.config.api.retry_attempts;
        this.client = axios.create({
            baseURL: this.config.api.base_url,
            timeout: this.config.api.timeout,
            headers: getAuthHeaders(this.config),
        });
        this.setupInterceptors();
    }
    /**
     * 设置请求和响应拦截器
     */
    setupInterceptors() {
        // 请求拦截器
        this.client.interceptors.request.use(config => {
            console.log(`[API] ${config.method?.toUpperCase()} ${config.url}`);
            return config;
        }, error => {
            console.error('[API] Request error:', error);
            return Promise.reject(error);
        });
        // 响应拦截器
        this.client.interceptors.response.use(response => {
            console.log(`[API] Response: ${response.status} ${response.config.url}`);
            return response;
        }, error => {
            console.error('[API] Response error:', error.message);
            return Promise.reject(error);
        });
    }
    /**
     * 通用GET请求
     */
    async get(url, params) {
        return this.requestWithRetry(() => this.client.get(url, { params }));
    }
    /**
     * 通用POST请求
     */
    async post(url, data, params) {
        // 合并默认URL参数
        const mergedParams = {
            ...getUrlParams(this.config),
            ...params,
        };
        return this.requestWithRetry(() => this.client.post(url, data, { params: mergedParams }));
    }
    /**
     * 带重试机制的请求执行
     */
    async requestWithRetry(requestFn) {
        let lastError;
        for (let attempt = 1; attempt <= this.maxRetries; attempt++) {
            try {
                const response = await requestFn();
                // 检查业务层面的错误
                if (response.data.err_no !== 0) {
                    throw new Error(`API Error: ${response.data.err_msg} (code: ${response.data.err_no})`);
                }
                return response.data;
            }
            catch (error) {
                lastError = error;
                console.warn(`[API] Attempt ${attempt}/${this.maxRetries} failed:`, error);
                // 如果不是最后一次尝试，等待后重试
                if (attempt < this.maxRetries) {
                    await this.delay(this.retryDelay * attempt);
                }
            }
        }
        throw new Error(`Request failed after ${this.maxRetries} attempts: ${lastError.message}`);
    }
    /**
     * 延迟函数
     */
    delay(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }
    /**
     * 构建查询参数
     */
    static buildParams(params) {
        const cleanParams = {};
        Object.entries(params).forEach(([key, value]) => {
            if (value !== undefined && value !== null && value !== '') {
                cleanParams[key] = value;
            }
        });
        return cleanParams;
    }
    /**
     * 格式化错误信息
     */
    static formatError(error) {
        if (error.response) {
            return `HTTP ${error.response.status}: ${error.response.statusText}`;
        }
        else if (error.request) {
            return 'Network error: No response received';
        }
        else {
            return `Request error: ${error.message}`;
        }
    }
}
// 导出单例实例
export const apiClient = new JuejinApiClient();
